@@ -360,11 +360,11 @@ def profile_connections (data_tweets):
                 if not screen_name_mention1:
                     screen_name_mention.append(["Empty"])
                 else:
-                    screen_name_mention1 = re.sub("[\[\]\'\"!@?¿#$:….,]", '',str(screen_name_mention1)).split(", ")  
                     if(data_tweets["text"][i].startswith('RT',0)):
+                        screen_name_mention1 = re.sub("[\[\]\'\"!@?¿#$:….]", '',str(screen_name_mention1)).split(", ") 
                         screen_name_mention.append([screen_name_mention1[0]])                    
                     else:
-                        screen_name_mention.append(re.sub("[\[\]\'\"!@?¿#$:….,]", '',str(screen_name_mention1)).split(" "))
+                        screen_name_mention.append(re.sub("[\[\]\'\"!@?¿#$:….]", '',str(screen_name_mention1)).split(", "))
             else:
                 screen_name_mention.append(["Empty"])
     data["screen_name"] = screen_name    
@@ -505,7 +505,7 @@ def graph_features(DiG,df):
     output = merge_dataframes(output, df_nodes_self_loop_iteration)
     return output
 
-def user_timeline_extraction (node_list, consumerKey, consumerSecret,accessToken,accessTokenSecret, iter_tw,save_by, ruta_fichero, nombre_fichero):
+def user_timeline_extraction (node_list, consumerKey, consumerSecret,accessToken,accessTokenSecret, iter_tw=1,save_by=300, filepath, filename):
     # get_user Returns the 20 most recent statuses posted from the authenticating user or the user specified. 
     profile_iter = save_by
     k=0
@@ -575,17 +575,17 @@ def user_timeline_extraction (node_list, consumerKey, consumerSecret,accessToken
                               "favourites_count": "favorites_count",
                               "full_text": "text"})
         df2 = metadata_extraction(df)
-        if path.exists(ruta_fichero + nombre_fichero):
-          df2.to_csv(nombre_fichero,mode="a", index=False, header=False)
+        if path.exists(filepath + filename):
+          df2.to_csv(filename,mode="a", index=False, header=False)
         else:
-          df2.to_csv(nombre_fichero,index=False)         
+          df2.to_csv(filename,index=False)         
         if len(list_users_noInfo) !=0:
           nodes = pd.DataFrame()
           nodes["nodos"] = list_users_noInfo
-          if path.exists(ruta_fichero+"node_without_"+nombre_fichero):
-            nodes.to_csv("node_without_"+ nombre_fichero, mode ="a",index=False, header=False)          
+          if path.exists(filepath+"node_without_"+filename):
+            nodes.to_csv("node_without_"+ filename, mode ="a",index=False, header=False)          
           else:
-             nodes.to_csv("node_without_"+ nombre_fichero, index=False)
+             nodes.to_csv("node_without_"+ filename, index=False)
         k = k+profile_iter        
         if ultima ==1:          
           ultima =2
@@ -743,7 +743,7 @@ def text_tweets_features_extraction(data_tweets):
 			#Found if tweets are the same day
 			#Second condition if all the tweets are the same day          
 			if((tweets_date[j-1].date() == tweets_date[j].date()) & (j!=(len(text_tweets_user)-1))):                            
-				num_tw_day = num_tw_day + 1
+				
 				#Create a list with the tweets of a day
 				text_tweets_day.append({"text": text_tweets_user[j-1],"lang": lang_text_tweet[j-1],"RT": is_RT_list[j-1]})
 				text_tweets_day.append({"text": text_tweets_user[j],"lang": lang_text_tweet[j], "RT": is_RT_list[j]})              
@@ -756,7 +756,7 @@ def text_tweets_features_extraction(data_tweets):
 						text_tweets_day.append({"text": text_tweets_user[j],"lang": lang_text_tweet[j], "RT": is_RT_list[j]})              
 						tweet_date_day.append(tweets_date[j-1])
 						tweet_date_day.append(tweets_date[j])
-						num_tw_day = num_tw_day + 1                      
+						                     
 						text_tweets_days.append([i for n, i in enumerate(text_tweets_day) if i not in text_tweets_day[n + 1:]] )
 						tweet_date_days.append(list(dict.fromkeys(tweet_date_day)))
 						num_tw_days.append(len(list(dict.fromkeys(tweet_date_day))))
@@ -879,6 +879,7 @@ def advanced_features(data):
     data["follow_rate"] = 0
     data["max_tw_day"] = 0
     data["index_max_day_tw"]=0
+    data["possible_duplicates"] = 0
     for i in range(len(data)):
         if (len(str(data.iloc[i]["screen_name"])) > 4):
             if((data.iloc[i]["screen_name"][-1].isdigit()) & (data.iloc[i]["screen_name"][-2].isdigit()) & 
@@ -903,20 +904,32 @@ def advanced_features(data):
             data["max_ret_tw"].iloc[i] = 0
         else:
             data["max_ret_tw"].iloc[i] = max((data["retweet_tweets_list"].iloc[i]))
-
+          
+        if i < len(data):
+          possible_duplicated = []
+          for j in range(len(data)):
+            if j !=i:
+              list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
+              list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
+              distance, path = fastdtw(list_dtw_1/max(max(list_dtw_1),max(list_dtw_2)),list_dtw_2/max(max(list_dtw_1),max(list_dtw_2)))
+              if (distance < 1.5):
+                possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
+        if len(possible_duplicated)==0:
+          possible_duplicated.append({"screen_name": "Empty", "distance": "Empty"})           
+        data["possible_duplicates"].iloc[i] = possible_duplicated
         data["max_tw_day"].iloc[i] =max((data["tw_per_day_list"].iloc[i]))
         data["days_with_tw"].iloc[i] = len(data["tw_per_day_list"].iloc[i])
         data["index_max_day_tw"].iloc[i] = data["tw_per_day_list"].iloc[i].index(data["max_tw_day"].iloc[i])
-        data["follow_rate"][data["followers"]== 0] = np.median(data["follow_rate"])
-        data["num_userTw"]= data["num_ownTw"] + data["RT"]
-        data["tw_RT_rate"] = ((((data["num_ownTw"]))) / (data["num_userTw"])).replace([np.inf, -np.inf], np.nan).fillna(0)
-        data["sum_fav_RT_ownTw"] = (data["retweet_tweets_count"] + data["favorite_tweets_count"]).replace([np.inf, -np.inf], np.nan).fillna(0)
-        data["iter_fav_RT_rate"] = (data["sum_fav_RT_ownTw"] / data["num_ownTw"]).replace([np.inf, -np.inf], np.nan).fillna(0)
-        data["tw_year_rate"] = ((data["statuses_count"]) / (data["twitter_years"])).replace([np.inf, -np.inf], np.nan).fillna(0)
-        data["time_tw_rate"] = (data["time_btw_tw"] / data["num_userTw"]).replace([np.inf, -np.inf], np.nan).fillna(0)
-        data["seasonality"] = data["seasonality"].replace([np.inf, -np.inf], np.nan).fillna(0)
-        data["fake_sum"] = data["default_profile_image"] + data["biography_profile"]+ data["screen_name_bot"]
-        data["fake_rate"] = ((data["tweets_url"] + data["mentions"]) /data["num_ownTw"]).replace([np.inf, -np.inf], np.nan).fillna(0)
+    data["follow_rate"][data["followers"]== 0] = np.median(data["follow_rate"])
+    data["num_userTw"]= data["num_ownTw"] + data["RT"]
+    data["tw_RT_rate"] = ((((data["num_ownTw"]))) / (data["num_userTw"])).replace([np.inf, -np.inf], np.nan).fillna(0)
+    data["sum_fav_RT_ownTw"] = (data["retweet_tweets_count"] + data["favorite_tweets_count"]).replace([np.inf, -np.inf], np.nan).fillna(0)
+    data["iter_fav_RT_rate"] = (data["sum_fav_RT_ownTw"] / data["num_ownTw"]).replace([np.inf, -np.inf], np.nan).fillna(0)
+    data["tw_year_rate"] = ((data["statuses_count"]) / (data["twitter_years"])).replace([np.inf, -np.inf], np.nan).fillna(0)
+    data["time_tw_rate"] = (data["time_btw_tw"] / data["num_userTw"]).replace([np.inf, -np.inf], np.nan).fillna(0)
+    data["seasonality"] = data["seasonality"].replace([np.inf, -np.inf], np.nan).fillna(0)
+    data["fake_sum"] = data["default_profile_image"] + data["biography_profile"]+ data["screen_name_bot"]
+    data["fake_rate"] = ((data["tweets_url"] + data["mentions"]) /data["num_ownTw"]).replace([np.inf, -np.inf], np.nan).fillna(0)
     data = trend_tweets_features(data) 
         
     return data
@@ -1037,64 +1050,62 @@ def filter_possible_irregular_profiles(data_join):
 
     return data_join
 
-def NLP_analysis(data, keyword_corpus, hashtag_corpus):
-    tweets_df = pd.DataFrame({'text':data["text_tweets_days_list"],'RT':data["RT_days_list"],'lang': data["lang_days_list"]})
-    tweets_dff = pd.DataFrame()
-    for i in range(len(tweets_df)):
-      aux = pd.DataFrame({'text':tweets_df["text"].iloc[i],'RT':tweets_df["RT"].iloc[i],'lang': tweets_df["lang"].iloc[i]})
-      tweets_dff = tweets_dff.append(aux, ignore_index=False).reset_index(drop=True)
-    tweets_df = tweets_df[tweets_df["RT"] !=2].reset_index(drop=True)
-    tweets_df = sentiental_analysis_features(tweets_df)
-    sample_data_tokenized = tokenize_tweets(tweets_df)
-    tweets_df = keyword_score(tweets_df, sample_data_tokenized,keyword_corpus)
-    tweets_df = hashtag_score(tweets_df,hashtag_corpus)	
-    tweets_df["terrorist_suggestion"] = 0
-    tweets_df["terrorist_suggestion"][(tweets_df["hashtag_score"]>0) | (tweets_df["keyword_score"]>0)] = 1
-    tweets_irr_words = len(tweets_df[tweets_df["terrorist_suggestion"]==1])
-    if len(tweets_df[tweets_df["terrorist_suggestion"]==1]) == 0:
-      data["per_hash_key"] = 0
-      data["negative_opinion"] = 0
-      data["negative_fact"] = 0
-      data["neutral"] = 0
-      data["positive_fact"] = 0
-      data["positive_opinion"] = 0
-    else:
-      data["per_hash_key"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==1]) / len(tweets_df))
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]>=0.6)]= 2
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]<0.6)]= 3
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < 0.1) & (tweets_df["polarity"] > -0.1)]= 4
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]<0.6)]= 5
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]>=0.6)]= 6
-      if len(tweets_df[tweets_df["terrorist_suggestion"]==2]) == 0:
-        data["negative_opinion"] = 0
-      else:
-        data["negative_opinion"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==2]) / tweets_irr_words)
-      if len(tweets_df[tweets_df["terrorist_suggestion"]==3]) == 0:
-        data["negative_fact"] = 0
-      else:
-        data["negative_fact"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==3]) / tweets_irr_words)
-      if len(tweets_df[tweets_df["terrorist_suggestion"]==4]) == 0:
-        data["neutral"] = 0
-      else:
-        data["neutral"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==4]) / tweets_irr_words)
-      if len(tweets_df[tweets_df["terrorist_suggestion"]==5]) == 0:
-        data["positive_fact"] = 0
-      else:
-        data["positive_fact"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==5]) / tweets_irr_words)
-      if len(tweets_df[tweets_df["terrorist_suggestion"]==6]) == 0:
-        data["positive_opinion"] = 0
-      else:
-        data["positive_opinion"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==6]) / tweets_irr_words)
-        
-      
-    return data,tweets_df
+def NLP_analysis(data, keyword_corpus, hashtag_corpus):    
+	aux = pd.DataFrame({'text':data["text_tweets_days_list"],'RT':data["RT_days_list"],'lang': data["lang_days_list"]})
+	tweets_df = pd.DataFrame()
+	for i in range(len(aux)):
+		aux2 = pd.DataFrame({'text':aux["text"].iloc[i],'RT':aux["RT"].iloc[i],'lang': aux["lang"].iloc[i]})
+		tweets_df = tweets_df.append(aux2, ignore_index=False).reset_index(drop=True)
+	tweets_df = tweets_df[tweets_df["RT"] !=2].reset_index(drop=True)
+	tweets_df = sentiental_analysis_features(tweets_df)
+	sample_data_tokenized = tokenize_tweets(tweets_df)
+	tweets_df = keyword_score(tweets_df, sample_data_tokenized,keyword_corpus)
+	tweets_df = hashtag_score(tweets_df,hashtag_corpus)	
+	tweets_df["terrorist_suggestion"] = 0
+	tweets_df["terrorist_suggestion"][(tweets_df["hashtag_score"]>0) | (tweets_df["keyword_score"]>0)] = 1
+	tweets_irr_words = len(tweets_df[tweets_df["terrorist_suggestion"]==1])
+	if len(tweets_df[tweets_df["terrorist_suggestion"]==1]) == 0:
+		data["per_hash_key"] = 0
+		data["negative_opinion"] = 0
+		data["negative_fact"] = 0
+		data["neutral"] = 0
+		data["positive_fact"] = 0
+		data["positive_opinion"] = 0
+	else:
+		data["per_hash_key"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==1]) / len(tweets_df))
+		tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]>=0.6)]= 2
+		tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]<0.6)]= 3
+		tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < 0.1) & (tweets_df["polarity"] > -0.1)]= 4
+		tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]<0.6)]= 5
+		tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]>=0.6)]= 6
+		if len(tweets_df[tweets_df["terrorist_suggestion"]==2]) == 0:
+			data["negative_opinion"] = 0
+		else:
+			data["negative_opinion"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==2]) / tweets_irr_words)
+		if len(tweets_df[tweets_df["terrorist_suggestion"]==3]) == 0:
+			data["negative_fact"] = 0
+		else:
+			data["negative_fact"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==3]) / tweets_irr_words)
+		if len(tweets_df[tweets_df["terrorist_suggestion"]==4]) == 0:
+			data["neutral"] = 0
+		else:
+			data["neutral"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==4]) / tweets_irr_words)
+		if len(tweets_df[tweets_df["terrorist_suggestion"]==5]) == 0:
+			data["positive_fact"] = 0
+		else:
+			data["positive_fact"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==5]) / tweets_irr_words)
+		if len(tweets_df[tweets_df["terrorist_suggestion"]==6]) == 0:
+			data["positive_opinion"] = 0
+		else:
+			data["positive_opinion"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==6]) / tweets_irr_words)
+	return data,tweets_df
 	
-def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessToken,accessTokenSecret,n_groups,save_by,ruta_fichero,nombre_fichero):
+def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessToken,accessTokenSecret,n_groups,save_by,filepath,filename):
 	df_evaluate = df[(df["evaluate_NLP"] !="No") & (df["evaluate_NLP"] !="More Data")]
 	node_list_moreData = list(df[df["evaluate_NLP"]=="More Data"]["screen_name"])
 	if len(node_list_moreData) >0:
-		df_metadata_moreData, list_more_Data_noData = user_timeline_extraction(node_list_moreData,consumerKey,consumerSecret,accessToken,accessTokenSecret,n_groups,save_by,ruta_fichero, nombre_fichero)
-		df_metadata_moreData = pd.read_csv(ruta_fichero + nombre_fichero)
+		df_metadata_moreData, list_more_Data_noData = user_timeline_extraction(node_list_moreData,consumerKey,consumerSecret,accessToken,accessTokenSecret,n_groups,save_by,filepath, filename)
+		df_metadata_moreData = pd.read_csv(filepath + filename)
 		df_metadata_moreData = text_to_list_features_conversor(df_metadata_moreData)
 		df_metadata_moreData = advanced_features(df_metadata_moreData)
 		df_metadata_moreData = merge_dataframes(df_metadata_moreData,df_graph_features)
@@ -1104,53 +1115,49 @@ def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessT
 	return df_evaluate
 	
 def text_to_list_features_conversor (data):
-    for index, row in data.iterrows():
-        print("Nuevo usuario")
-        print (index)
-        data["favorite_tweets_list"].iloc[index] = ast.literal_eval(data.loc[index, "favorite_tweets_list"])
-        data["retweet_tweets_list"].iloc[index] = ast.literal_eval(data.loc[index,"retweet_tweets_list"])
-        data["tw_per_day_list"].iloc[index] = ast.literal_eval(data.loc[index,"tw_per_day_list"])
-        data["text_tweets_days_list"].iloc[index] = data.loc[index,"text_tweets_days_list"].replace('"', "'").replace("], [","?#??").strip("[]'").split("?#??")
-        
-        list_day = []                                                                                
-        list_text_tweets_day = []
-        list_tweet_date_days = []
-        list_RT_days = []
-        list_lang_days = []
-        text_days = data.loc[index,"tw_day_list"].strip("][").replace(")", ")??").split("??, ")
-        
-        tweet_date_days = data.loc[index,"tweet_date_days_list"].replace("], [","???").split("???")
-        RT_days = data.loc[index, "RT_days_list"].strip("[]").split("], [")
-        lang_days = data.loc[index, "lang_days_list"].strip("[]").split("], [")
-        
-        for i in range(len(text_days)):
-            tweet_date_day = tweet_date_days[i].replace("), ",")??").strip("[]").split("??")
-            text_tweets_day = data["text_tweets_days_list"].iloc[index][i].replace('"', "'").replace("¿¿??##%', '","¿¿??##%").strip("[]'").split("¿¿??##%")[:-1]
-            if len(RT_days[i])==1:
-                 RT_day = list(RT_days[i])
-            else:
-                RT_day = list(ast.literal_eval(str(RT_days[i])))
-            lang_day = list(ast.literal_eval(lang_days[i]))
-            list_tweet_date_day = []
-            for j in range(len(tweet_date_day)):                
-                if(tweet_date_day == ["'Empty'"]):
-                    list_tweet_date_day.append("Empty")
-                else:
-                    list_tweet_date_day.append(datetime.strptime(tweet_date_day[j],"Timestamp('%Y-%m-%d %H:%M:%S%z', tz='UTC')"))    
-                
-            if i == len(text_days)-1:
-                text_days[i] = text_days[i].replace("??","")
-            list_day.append(datetime.strptime(text_days[i],'datetime.date(%Y, %m, %d)' ).date())
-            list_text_tweets_day.append(text_tweets_day)
-            list_tweet_date_days.append(list_tweet_date_day)
-            list_RT_days.append(RT_day)
-            list_lang_days.append(lang_day)
-        data["text_tweets_days_list"].iloc[index] = list_text_tweets_day
-        data["tw_day_list"].iloc[index]= list_day
-        data["tweet_date_days_list"].iloc[index]= list_tweet_date_days
-        data["RT_days_list"].iloc[index] = list_RT_days
-        data["lang_days_list"].iloc[index] = list_lang_days
-    return data
+	for index, row in data.iterrows():
+		print("Nuevo usuario")
+		print (index)
+		data["favorite_tweets_list"].iloc[index] = ast.literal_eval(data.loc[index, "favorite_tweets_list"])
+		data["retweet_tweets_list"].iloc[index] = ast.literal_eval(data.loc[index,"retweet_tweets_list"])
+		data["tw_per_day_list"].iloc[index] = ast.literal_eval(data.loc[index,"tw_per_day_list"])
+		data["text_tweets_days_list"].iloc[index] = data.loc[index,"text_tweets_days_list"].replace('"', "'").replace("], [","?#??").strip("[]'").split("?#??")
+		list_day = []                                                                                
+		list_text_tweets_day = []
+		list_tweet_date_days = []
+		list_RT_days = []
+		list_lang_days = []
+		text_days = data.loc[index,"tw_day_list"].strip("][").replace(")", ")??").split("??, ")
+		tweet_date_days = data.loc[index,"tweet_date_days_list"].replace("], [","???").split("???")
+		RT_days = data.loc[index, "RT_days_list"].strip("[]").split("], [")
+		lang_days = data.loc[index, "lang_days_list"].strip("[]").split("], [")
+		for i in range(len(text_days)):
+			tweet_date_day = tweet_date_days[i].replace("), ",")??").strip("[]").split("??")
+			text_tweets_day = data["text_tweets_days_list"].iloc[index][i].replace('"', "'").replace("¿¿??##%', '","¿¿??##%").strip("[]'").split("¿¿??##%")[:-1]
+			RT_day = list(ast.literal_eval(str(RT_days[i])))
+			if len(lang_days[i])<6:
+				lang_day = [lang_days[i]]
+			else:
+				lang_day = list(ast.literal_eval(lang_days[i]))
+			list_tweet_date_day = []
+			for j in range(len(tweet_date_day)):                
+				if(tweet_date_day == ["'Empty'"]):
+					list_tweet_date_day.append("Empty")
+				else:
+					list_tweet_date_day.append(datetime.strptime(tweet_date_day[j],"Timestamp('%Y-%m-%d %H:%M:%S%z', tz='UTC')"))    
+			if i == len(text_days)-1:
+				text_days[i] = text_days[i].replace("??","")
+			list_day.append(datetime.strptime(text_days[i],'datetime.date(%Y, %m, %d)' ).date())
+			list_text_tweets_day.append(text_tweets_day)
+			list_tweet_date_days.append(list_tweet_date_day)
+			list_RT_days.append(RT_day)
+			list_lang_days.append(lang_day)
+		data["text_tweets_days_list"].iloc[index] = list_text_tweets_day
+		data["tw_day_list"].iloc[index]= list_day
+		data["tweet_date_days_list"].iloc[index]= list_tweet_date_days
+		data["RT_days_list"].iloc[index] = list_RT_days
+		data["lang_days_list"].iloc[index] = list_lang_days
+	return data
 
 	
 def evaluate_NLP_profiles(data,keyword_corpus, hashtag_corpus):	
